using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using Discord.WebSocket;
using Discord;
using Discord.Commands;
using System;
using System.Threading.Tasks;
using spike.Utility.FileManaging;
using spike.Data.PersistentData;
using Newtonsoft.Json;
using template.Modules;
using template.Embed;


namespace spike.Modules
{
    [Group("move")]
    public class SpeakModule : ModuleBase<SocketCommandContext>
    {
        private readonly DiscordSocketClient _client;
        public static ListMessageRef messageRef = null;
        public SpeakModule(DiscordSocketClient client)
        {
            _client = client;
        }
        private String SpellCheck(string input)
        {
            var spellcheck = new Dictionary<string, string>(){
                {"pl.*[ayer|mov|moves]", "Player Move"},
                {"cu.*[s|moves|move|m]", "Custom Move"},
                {"ad.*[v|tage|tages]", "Advantage"},
                {"di.*[s|tage|tages]", "Disadvantage"},
                {".*", null}
            };
            var results = from found in spellcheck
                          where Regex.Match(input, found.Key, RegexOptions.Singleline).Success
                          select found;
            return results.ElementAt(0).Value;
        }
        private bool SearchMove(Move input, string search)
        {
            if (input.keyword.ToLower().Contains(search.ToLower()))
                return true;
            if (input.longName.ToLower().Contains(search.ToLower()))
                return true;
            if (input.rollDesc.ToLower().Contains(search.ToLower()))
                return true;
            if (input.longDesc.ToLower().Contains(search.ToLower()))
                return true;
            if (input.successResult != null)
            {
                if (input.successResult.ToLower().Contains(search.ToLower()))
                    return true;
                if (input.complicationResult.ToLower().Contains(search.ToLower()))
                    return true;
                if (input.failResult.ToLower().Contains(search.ToLower()))
                    return true;
            }
            return false;
        }
        [Command("")]
        [Priority(1)]
        public async Task MainMoveCommand(String move, int b = 0)
        {
            var moveList = FileManager.PopulateList(Context.Guild.Id, MoveTypes.all);
            var result = moveList.Find(x => x.keyword.ToLower().Equals(move));
            if (result == null)
            {
                await Context.Channel.SendMessageAsync("Could not find a move matching *" + move + "*. Type `!move list` followed by either `player`, `advantage`, `disadvantage` or `custom` to see what moves are available.");
            }
            else
            {
                EmbedBuilder builder = new EmbedBuilder();
                builder.WithTitle(result.longName);
                builder.WithDescription(result.longDesc);
                builder.WithColor(150, 140, 106);
                if (result.successResult != null)
                {
                    Random rnd = new Random();
                    var d1 = rnd.Next(1, 10);
                    var d2 = rnd.Next(1, 10);
                    b = Math.Max(Math.Min(b,99),-99);

                    var roll = d1 + d2 + b;
                    builder.AddField("Roll", d1 + "+" + d2, true);
                    builder.AddField("Bonus", b, true);
                    builder.AddField("Total", d1 + d2 + b, true);
                    builder.WithThumbnailUrl("https://kult.tools/diceGen/index.php?d1=" + d1 + "&d2=" + d2 + "&b=" + b);

                    var outResult = "";
                    if (roll <= 9)
                    {
                        outResult = result.failResult;
                    }
                    else if (roll <= 14)
                    {
                        outResult = result.complicationResult;
                    }
                    else
                    {
                        outResult = result.successResult;
                    }
                    builder.AddField("Result", outResult, false);
                }
                await Context.Channel.SendMessageAsync("", false, builder.Build());
            }
        }
        [Command("list")]
        [Priority(2)]
        public async Task moveListCommand(string type, int page = 1)
        {
            string ListType = SpellCheck(type);
            if (ListType == null)
            {
                await Context.Channel.SendMessageAsync("There is no list of the type *" + type + "*. Type `!move list` followed by either `player`, `advantage`, `disadvantage` or `custom`.");
                return;
            }
            List<Move> moveList = FileManager.PopulateList(Context.Guild.Id, MoveTypes.all);
            var result = moveList.FindAll(x => x.type.Equals(ListType));
            if (result.Count < 1)
            {
                await Context.Channel.SendMessageAsync("The list is empty.");
                return;
            }
            var embedMsg = EmbedFactory.MakeListEmbed(ListType, result, page, Context.Guild.Id);
            var message = await Context.Channel.SendMessageAsync("", false, embedMsg);
            if (result.Count > 30)
            {
                await message.AddReactionsAsync(new Emoji[] { new Emoji("⬅️"), new Emoji("➡️") });
                messageRef = new ListMessageRef(ListType, page, message.Id, message, Context.User.Id, Context.Guild.Id);
            }
        }
        [Command("search")]
        [Priority(2)]
        public async Task moveListCommand(string search)
        {
            var moveList = FileManager.PopulateList(Context.Guild.Id, MoveTypes.all);
            var result = moveList.FindAll(x => SearchMove(x, search));
            if (result.Count > 0)
            {
                EmbedBuilder builder = new EmbedBuilder();
                builder.WithTitle("Searching for *" + search + "*.");
                int ItemsPerPage = 30;

                string items1 = "";
                string items2 = "";
                string items3 = "";
                for (var i = 0; i < ItemsPerPage && i < result.Count; i++)
                {
                    items1 += "` `" + result[i].longName + "\n";
                    items2 += "`" + result[i].keyword.ToLower() + "`\n";
                    items3 += "` `" + result[i].type + "\n";
                }
                builder.AddField("Move", items1, true);
                builder.AddField("Keyword", items2, true);
                builder.AddField("Roll", items3, true);
                if (result.Count > ItemsPerPage)
                {
                    builder.WithDescription("Found " + result.Count + " moves matching *" + search + ".");
                }
                builder.WithColor(150, 140, 106);

                var embedMsg = builder.Build();
                await Context.Channel.SendMessageAsync("", false, embedMsg);
            }
            else
            {
                await Context.Channel.SendMessageAsync("Found no moves matching *" + search + "*.");
            }
        }
        [Command("help")]
        [Priority(2)]
        public async Task moveHelpCommand(string item = "")
        {
            if (item == "")
            {
                var outText = "```md\n" +
                                "# MoveBot 1.0 - Usage guide````!move help` - Display this text\n" +
                                "`!move xxx` - Roll for the move with the keyword *xxx*.\n" +
                                "`!move xxx #` - Roll for the move with the keyword *xxx* with a number between -99 and 99 instead of *#*.\n" +
                                "`!move list player` - List player moves and their keywords.\n" +
                                "`!move list advantage` - List advantages and their keywords.\n" +
                                "`!move list disadvantage` - List disadvantages and their keywords.\n" +
                                "`!move list custom` - List custom moves and their keywords.\n" +
                                "`!move help xxx` - Get information for the move with the keyword *xxx*.\n" +
                                "To roll for a move you write the keyword instead of *xxx*. These keywords can be found in their respective lists. For instance typing *`!move kit 2`* would roll for ***Keep it Together*** with a bonus of ***+2***.\n" +
                                "```md\n" +
                                "# Custom moves````!move add` - Show link to the web interface for creating a new move.\n" +
                                "`!move add data` - Add a new custom move, the data is generated form the link in the command above.\n" +
                                "`!move delete xxx` - Delete custom move with the keyword *xxx*.\n";
                await Context.Channel.SendMessageAsync(outText);
            }
            else
            {
                var moveList = FileManager.PopulateList(Context.Guild.Id, MoveTypes.all);
                var result = moveList.Find(x => x.keyword.ToLower().Equals(item));
                if (result == null)
                {
                    await Context.Channel.SendMessageAsync("Could not find a move matching *" + item + "*. Type `!move list` followed by either `player`, `advantage`, `disadvantage` or `custom` to see what moves are available.");
                }
                else
                {
                    EmbedBuilder builder = new EmbedBuilder();
                    builder.WithTitle(result.longName);
                    builder.WithDescription(result.longDesc);
                    if (result.successResult != "")
                    {
                        builder.AddField("15+", result.successResult, false);
                        builder.AddField("10-14", result.complicationResult, false);
                        builder.AddField("-9", result.failResult, false);
                    }
                    builder.WithColor(150, 140, 106);
                    await Context.Channel.SendMessageAsync("", false, builder.Build());
                }
            }
        }

        [RequireUserPermission(GuildPermission.Administrator, Group = "Permission")]
        [RequireOwner(Group = "Permission")]
        [Command("add")]
        [Priority(2)]
        public async Task CustomAddCommand([Remainder] string input)
        {
            var moveList = FileManager.PopulateList(Context.Guild.Id, MoveTypes.all);
            Move content = JsonConvert.DeserializeObject<Move>(input);
            content.type = "Custom Move";
            List<Move> result = moveList.FindAll(x => x.keyword.Equals(content.keyword.ToLower()));
            if (result.Count > 0)
            {
                await Context.Channel.SendMessageAsync("You already have a move with the keyword `" + content.keyword + "` please change it to something else.");
            }
            else
            {
                result = moveList.FindAll(x => x.type.Equals("Custom Move"));
                result.Add(content);
                MoveList output = new MoveList();
                output.moves = result;
                FileManager.SaveMoves(output, MoveTypes.custom, Context.Guild.Id);
                await Context.Channel.SendMessageAsync("Added a new move called *" + content.longName + "* to the Custom Moves list.");
            }
        }

        [RequireUserPermission(GuildPermission.Administrator, Group = "Permission")]
        [RequireOwner(Group = "Permission")]
        [Command("delete")]
        [Priority(2)]
        public async Task CustomDeleteCommand(string input)
        {
            List<Move> moveList = FileManager.PopulateList(Context.Guild.Id, MoveTypes.custom);
            Move result = moveList.Find(x => x.keyword.Equals(input.ToLower()));
            if (result != null)
            {
                moveList.Remove(result);
                MoveList output = new MoveList();
                output.moves = moveList;
                FileManager.SaveMoves(output, MoveTypes.custom, Context.Guild.Id);
            }
            else
            {
                await Context.Channel.SendMessageAsync("Could not find a move matching *" + input + "*. Type `!move list` followed by either `player`, `advantage`, `disadvantage` or `custom` to see what moves are available.");
            }
        }

        [Command("update")]
        [Priority(2)]
        public Task UpdateMoveListCommand()
        {
            MoveListManager.Instance.UpdateMoveList();
            return Task.CompletedTask;
        }
    }
}
