using System.Data.Common;
using System.Collections.Immutable;
using System.Xml.Linq;
using System.Linq;
using System.Reflection.Metadata;
using spike.Utility.FileManaging;
using System;
using System.Reflection;
using System.Threading.Tasks;
using Microsoft.Extensions.DependencyInjection;
using Discord;
using Discord.Commands;
using Discord.WebSocket;
using Microsoft.Extensions.Configuration;
using System.Collections.Generic;
using spike.Data.PersistentData;
using template.Embed;
using template.Modules;


namespace spike.Services
{
    public class CommandHandler
    {
        // setup fields to be set later in the constructor
        private readonly IConfiguration _config;
        private readonly CommandService _commands;
        private readonly DiscordSocketClient _client;
        private readonly IServiceProvider _services;

        public CommandHandler(IServiceProvider services)
        {
            // juice up the fields with these services
            // since we passed the services in, we can use GetRequiredService to pass them into the fields set earlier
            _config = services.GetRequiredService<IConfiguration>();
            _commands = services.GetRequiredService<CommandService>();
            _client = services.GetRequiredService<DiscordSocketClient>();
            _services = services;

            // take action when we execute a command
            _commands.CommandExecuted += CommandExecutedAsync;

            // take action when we receive a message (so we can process it, and see if it is a valid command)
            _client.MessageReceived += MessageReceivedAsync;
            _client.ReactionAdded += ReactionAdded_Event;
        }

        public async Task ReactionAdded_Event(Cacheable<IUserMessage, UInt64> cashedMessage, ISocketMessageChannel channel, SocketReaction reaction)
        {
            IUser user = reaction.User.IsSpecified ? reaction.User.Value : null;
            if (spike.Modules.SpeakModule.messageRef == null || user.IsBot)
                return;

            if (spike.Modules.SpeakModule.messageRef.msgID == cashedMessage.Id)
            {
                string ListType = spike.Modules.SpeakModule.messageRef.keyword;
                List<Move> moveList = FileManager.PopulateList(spike.Modules.SpeakModule.messageRef.guildID, MoveTypes.all);
                var result = moveList.FindAll(x => x.type.Equals(ListType));
                if (reaction.Emote.Name == "⬅️")
                {
                    int newPage = spike.Modules.SpeakModule.messageRef.page-1;
                    spike.Modules.SpeakModule.messageRef.page = Math.Max(0, newPage);
                }
                else if (reaction.Emote.Name == "➡️")
                {
                    int maxPages = (int)Math.Ceiling((float)result.Count/30.0);
                    int newPage = spike.Modules.SpeakModule.messageRef.page+1;
                    int selectedPage = Math.Min(maxPages,newPage);
                    spike.Modules.SpeakModule.messageRef.page = Math.Min(maxPages,newPage);
                }
                var embedMsg = EmbedFactory.MakeListEmbed(ListType, result, spike.Modules.SpeakModule.messageRef.page, spike.Modules.SpeakModule.messageRef.guildID);
                await spike.Modules.SpeakModule.messageRef.message.ModifyAsync(x => x.Embed = embedMsg);

                if (user != null)
                    await spike.Modules.SpeakModule.messageRef.message.RemoveReactionAsync(reaction.Emote, user);
            }
        }

        public async Task InitializeAsync()
        {
            // register modules that are public and inherit ModuleBase<T>.
            await _commands.AddModulesAsync(Assembly.GetEntryAssembly(), _services);
        }

        // this class is where the magic starts, and takes actions upon receiving messages
        public async Task MessageReceivedAsync(SocketMessage rawMessage)
        {
            // ensures we don't process system/other bot messages
            if (rawMessage.Source == MessageSource.System ||
                rawMessage.Source == MessageSource.Bot)
            {
                return;
            }

            SocketUserMessage message = rawMessage as SocketUserMessage;

            if (message.Source != MessageSource.User)
            {
                return;
            }
            // sets the argument position away from the prefix we set
            var argPos = 0;

            // get prefix from the configuration file
            char prefix = Char.Parse(_config["Prefix"]);

            var context = new SocketCommandContext(_client, message);
            // determine if the message has a valid prefix, and adjust argPos based on prefix

            if (message.HasCharPrefix(prefix, ref argPos))
            {
                await _commands.ExecuteAsync(context, argPos, _services);
                return;
            }
        }

        public async Task CommandExecutedAsync(Optional<CommandInfo> command, ICommandContext context, IResult result)
        {
            // if a command isn't found, log that info to console and exit this method
            if (!command.IsSpecified)
            {
                var test = command;
                Console.WriteLine($"Command failed to execute since it was not handled by any module..!");
                return;
            }


            // log success to the console and exit this method
            if (result.IsSuccess)
            {
                var commandName = command.IsSpecified ? command.Value.Name : "A command";
                Console.WriteLine($"Command {commandName} executed for -> {context.User.Username}");
                return;
            }

            else
            {
                Console.WriteLine($"An Error occurred: {result.ErrorReason}");
            }


            // failure scenario, let's let the user know
            await context.Channel.SendMessageAsync($"Sorry ... -> {result.ErrorReason}!");
        }
    }
}

